#ifndef _QCK_PLUGIN_H_
#define _QCK_PLUGIN_H_

#ifndef _WINDOWS__OS_
#if(defined(_WinNT_) || defined(WIN32) || defined(WIN64) || defined(_AMD64_))
#define _WINDOWS__OS_
#endif
#endif

#ifndef _UNIX__OS_
#if(defined(_UNIX_C_) || defined(_SUN_UX_) || defined(_LINUX_))
#define _UNIX__OS_
#endif
#endif

#ifdef _UNIX__OS_
typedef long long LongLong_t;
#elif defined(_WINDOWS__OS_)
typedef __int64 LongLong_t;
#endif

#ifdef QCKPLUGIN_DLL_SPEC
#undef QCKPLUGIN_DLL_SPEC
#endif

#ifdef USRPLUGIN_DLL_SPEC
#undef USRPLUGIN_DLL_SPEC
#endif

#ifdef _WINDOWS__OS_
#ifdef _FOR_QCKPLUGIN_BUILD_
#define QCKPLUGIN_DLL_SPEC __declspec (dllexport)
#define USRPLUGIN_DLL_SPEC __declspec (dllimport)
#else
#define QCKPLUGIN_DLL_SPEC __declspec (dllimport)
#define USRPLUGIN_DLL_SPEC __declspec (dllexport)
#endif
#else
#define QCKPLUGIN_DLL_SPEC
#define USRPLUGIN_DLL_SPEC
#endif

#ifdef __cplusplus

#define QCKPLUGIN_VER_STR "QckPlugin Framework v1.5.0 (May 28, 2009)"
#define QCKPLUGIN_APIVER 0x56559B6

#include "qckerror.h"

namespace NQckPlugin {

enum eInfoMode
{
  infoVERTEX = 0,
  infoWINDOW 
};

enum eNotificationTypes
{
 notifyWINDOW = 1,
 notifyPOINT,
 notifyDRAW,
 notifyQUERY,
 notifyDB,
 notifyPOLYAREA,
 notifyLAYERS
};

enum eDrawBoundaryType
{
  POINT = 0,
  OUTLINE,
  POLYGON  
};

enum eTextJustifyType
{
  justifyTOPLEFT = 0,
  justifyTOPCENTER = 10,
  justifyTOPRIGHT = 20,
  justifyMIDLEFT = 1,
  justifyMIDCENTER = 11,
  justifyMIDRIGHT = 21,
  justifyLOWLEFT = 2,
  justifyLOWCENTER = 12,
  justifyLOWRIGHT = 22
};

enum eMiscFlagsType
{
  nestingLevelALL = -1
};

enum eQckPluginArgType
{
  argtypeMENUID = 1
};

enum eCADFileType
{
  filetypeUNKNOWN = -1,
  filetypeGDSII = 1,
  filetypeOASIS, 
  filetypeOA
};

enum eMarkerShapeType
{
   markerShapeCROSS = 1,
   markerShapeDIAMOND,
   markerShapeX,
   markerShapeSQUARE,
   markerShapeTRIANGLE,
   markerShapeBOXEDX,
   markerShapeRTRIANGLE,
   markerShapeBOXEDCROSS,

   markerSizeDEFAULT = 3,
   markerWidthDEFAULT = 3
};

}

struct QCKPLUGIN_DLL_SPEC sQCKPLUGINARGS
{
  unsigned char mType;
  LongLong_t mArg; 
};

struct QCKPLUGIN_DLL_SPEC sQCKWINDOW
{
  double mMinX;  
  double mMinY;
  double mMaxX;
  double mMaxY;
};

struct QCKPLUGIN_DLL_SPEC sQCKPOLYWINDOW
{
  unsigned int mNumVert;
  double* mXY;
};

struct QCKPLUGIN_DLL_SPEC sQCKMARKER
{
  unsigned short mShape;
  unsigned short mRGB[3];
  unsigned short mSize;
  unsigned short mThickness;
  
  double mXY[2];

  sQCKMARKER();
  void Clear();
};

struct QCKPLUGIN_DLL_SPEC sQCKEXTRACTPARAMS
{
  const char* mOutputFileName;
  const char* mTopCellName;
  bool mClip;
};

struct QCKPLUGIN_DLL_SPEC sQCKEXTRACTWINDOW: public sQCKEXTRACTPARAMS
{
  sQCKWINDOW mWindow;
};

struct QCKPLUGIN_DLL_SPEC sQCKEXTRACTPOLY: public sQCKEXTRACTPARAMS
{
  sQCKPOLYWINDOW mPoly;
};

struct QCKPLUGIN_DLL_SPEC sQCKBOUNDARY
{
  const char* mCellName;
  unsigned short mLayer;
  unsigned short mDataType;
  unsigned int mNumVert;
  const double* mXY;
};

struct QCKPLUGIN_DLL_SPEC sQCKPATH
{
  char mType;
  const char* mCellName;
  unsigned short mLayer;
  unsigned short mDataType;
  unsigned int mNumVert;
  const double* mXY;
  double mWidth;
};

struct QCKPLUGIN_DLL_SPEC sTMATRIX
{
  double mTm[4];
  double mRowOffset[2];
  double mColOffset[2];
};

struct QCKPLUGIN_DLL_SPEC sTPARAMS
{
  char mMirror;
  double mScale;
  double mAngle;
};

struct QCKPLUGIN_DLL_SPEC sQCKSREF
{
  bool mMatrix;
  const char* mRefName;
  const char* mParentName;
  double mInsertion[2];
  double* mExtentsXY;
  union
  {
    sTMATRIX mTMatrix;
    sTPARAMS mTParams;
  };
};

struct QCKPLUGIN_DLL_SPEC sPARENTREF
{
  int mCellID;
  double mExtents[8];
};

struct QCKPLUGIN_DLL_SPEC sQCKHSREF: public sQCKSREF
{
  int mNumParents;
  sPARENTREF* mParentTree; 
};

struct QCKPLUGIN_DLL_SPEC sQCKAREF 
{
  bool mMatrix;
  const char* mRefName;
  const char* mParentName;
  unsigned int mSize[2];
  double mInsertion[2];
  double* mExtentsXY;
  union
  {
    sTMATRIX mTMatrix;
    sTPARAMS mTParams;
  };
};

struct QCKPLUGIN_DLL_SPEC sQCKHAREF: public sQCKAREF
{
  int mNumParents;
  sPARENTREF* mParentTree;
};

struct QCKPLUGIN_DLL_SPEC sQCKTEXT
{
  char mMirror;
  unsigned short mJustify;
  unsigned short mFontID;
  unsigned short mLayer;
  unsigned short mTextType;
  const char* mCellName;
  const char* mTextString;
  double mInsertion[2];
  double mScale;
  double mAngle;
  double* mExtentsXY;
};

struct QCKPLUGIN_DLL_SPEC sQCKVECTOR
{
  char mType;
  unsigned short mNestingLevel;
  union 
  {
    sQCKBOUNDARY mBoundary;
    sQCKPATH mPath;
    sQCKSREF mSref;
    sQCKAREF mAref;
    sQCKTEXT mText;
    sQCKHAREF mHAref;
    sQCKHSREF mHSref;
  };
};

struct QCKPLUGIN_DLL_SPEC sQCKINFOVECTOR : public sQCKVECTOR
{
  int mVertexID; 
};

struct QCKPLUGIN_DLL_SPEC sQCKQUERYSETTINGS
{
  bool mPathAsPath;
  bool mGetText;
  int mNestingLevel;
  sQCKWINDOW mWindow;

  sQCKQUERYSETTINGS();
  void Clear();
};

class QCKPLUGIN_DLL_SPEC IQckEventNotify
{
public:
  IQckEventNotify();
  virtual void OnDrawWindowRubberBand(const sQCKWINDOW& WindowExts);
  virtual void OnChangeViewWindow(const sQCKWINDOW& WindowExts);
  virtual void OnSelectPoint(double X, double Y);
  virtual void OnRedraw();
  virtual void OnGetDataVector(const sQCKVECTOR& VectorDatum);
  virtual void OnInfoObject(const sQCKINFOVECTOR& VectorDatum);
  virtual int PreFileOpen();
  virtual void PostFileOpen(const char* openedFileName, int fileType);
  virtual int PreCellOpen();
  virtual void PostCellOpen(const char* openedCellName);
  virtual void OnSelectPolygonArea(const sQCKPOLYWINDOW& polyExts);
  virtual void OnChangeLayers(bool showDatatypes);
  #ifdef _WINDOWS__OS_
  virtual void OnShowDialog(const bool show);
  #endif
};

class QCKPLUGIN_DLL_SPEC IQckWindowSrvc
{
public:
  virtual void SetWindowServiceNotify(void* PlgHandle, IQckEventNotify* Client) = 0;
  virtual void SelectPoint(void* PlgHandle, IQckEventNotify* Client) = 0;
  virtual void SetDrawRubberBand(bool OnOff) = 0;
  virtual void SetSelectObject(bool setOnOff) = 0;
  virtual void SetRubberBandPoint(double X, double Y) = 0;
  virtual int SetWindow(const sQCKWINDOW& windowExts, bool refreshView = true) = 0;
  virtual int SetExactWindow(const sQCKWINDOW& WindowExts, bool refreshView = true) = 0;
  virtual void GetCurrentWindow(sQCKWINDOW& WindowExts) = 0;
  virtual void SelectPolygonArea(void* PluginHandle, IQckEventNotify* EventHandler) = 0;
};

class QCKPLUGIN_DLL_SPEC IQckMenuSrvc
{
public:
  virtual int CreateMenuItem(void* PluginHandle, const char* itemName) = 0;
  virtual void* CreatePopupMenu(void* PluginHandle, const char* itemName, bool newMenu = false) = 0;
  virtual int AddPopupMenuItem(void* popupMenuHandle, const char* itemName) = 0;
  #ifdef _UNIX__OS_
  virtual void* GetAppShell() = 0;
  #endif
};

class QCKPLUGIN_DLL_SPEC IQckMarkerSrvc
{
public:
  virtual int DrawMarker(const sQCKMARKER& Marker) = 0;
  virtual int ZoomToMarker(const sQCKMARKER& Marker, double WindowSizeX, double WindowSizeY, bool refresh = true) = 0;
};

class QCKPLUGIN_DLL_SPEC IQckExtractSrvc
{
public:
  virtual int ExtractGDSII(const sQCKEXTRACTWINDOW& Params) = 0;
  virtual int ExtractGDSII(const sQCKEXTRACTPOLY& Params) = 0;
};

class QCKPLUGIN_DLL_SPEC IQckGraphicsSrvc
{
public:
  virtual void SetDrawServiceNotify(void* PlgHandle, IQckEventNotify* Client) = 0;
  virtual void Redraw(const bool forceredraw, bool zoomHome = false) = 0;
  virtual void SelectForegroundColor() = 0;
  virtual bool SelectColor(unsigned short R, unsigned short G, unsigned short B) = 0;
  virtual int DrawBoundary(const int Type,const double *UserUnits,const int NumVert,const bool DrawOutline) = 0; 
  virtual bool IsSmallObject(int numVertices, const double* xyCoords, int thresholdInPixels, sQCKWINDOW& newExtentsBox) = 0;
  virtual void SetLineSize(unsigned int lineSize) = 0;
  virtual double GetPixelSize() = 0;
};

class QCKPLUGIN_DLL_SPEC IQckQuerySrvc
{
public:
  virtual void Stop() = 0;
  virtual void SetQueryServiceNotify(void* PlgHandle, IQckEventNotify* Client) = 0;
  virtual void SetViewerInfoMode(bool setOnOff, int infoMode = 0) = 0;
  virtual void SetReferenceVectorMatrix(bool setOn) = 0;
  virtual int GetDataVector(IQckEventNotify* client, const sQCKQUERYSETTINGS* settings = 0) = 0;
  virtual int GetCellReferences(const char* CellName, int NestingLevel, IQckEventNotify* Client) = 0;
};

class QCKPLUGIN_DLL_SPEC IQckDatabaseSrvc
{
public:
  virtual void SetDatabaseServiceNotify(void* PlgHandle, IQckEventNotify* Client) = 0;

  virtual int SetViewCell(const char* cellName, bool refreshView = true) = 0;

  virtual int GetGrid(double& aBuffer) = 0;
  virtual int GetUnits(double& aBuffer, char* aStrBuffer = 0) = 0;
  virtual int GetCellID(const char* aCellName) = 0;
  virtual const char* GetCellName(int anID) = 0;
  virtual const char* GetViewCell() = 0;
  virtual int GetCellList(char*** aCellListPtr) = 0;
  virtual int GetCellRoot(char*** aCellListPtr) = 0;
  virtual int GetCellParents(const char* aCell, char*** aCellListPtr) = 0;
  virtual int GetCellChildren(const char* aCell, char*** aCellListPtr) = 0;
  virtual int GetCellExtents(const char* aCell, sQCKWINDOW& aBuffer) = 0;
  virtual int GetCellVertices(const char* aCell, LongLong_t& nBoundaries, LongLong_t& nPaths) = 0;   
  virtual int GetCellInfo(const char* aCell, LongLong_t& nBndrys, LongLong_t& nPaths, LongLong_t& nTexts, 
                          LongLong_t& nSrefs, LongLong_t& nArefs) = 0;
  virtual int GetLayerList(unsigned short** layerListPtr, unsigned short** dtListPtr) = 0;
  virtual int GetLayerList(char*** aStrListPtr) = 0;
  virtual int GetDatatypesForLayer(unsigned short aLayer, unsigned short** dtListPtr) = 0;

  static void FreeCellList(char** aList, int nListItems);
  static void FreeLayerList(unsigned short* layerList);
  static void FreeLayerList(char** aList, int nListItems); 
};

class QCKPLUGIN_DLL_SPEC IQckLayerSrvc
{
public:
  virtual void SetLayerServiceNotify(void* PlgHandle, IQckEventNotify* Client) = 0;
  virtual void SetLayerOnOff(unsigned short layer, unsigned short dataType, bool onOff = true, bool refresh = true) = 0;
  virtual void SetLayerOnOff(const char *layer, bool onOff = true, bool refresh = true) = 0;
  virtual void SetAllLayersOnOff(bool onOff = true, bool refresh = true) = 0;
  virtual bool GetLayerOnOff(unsigned short layer, unsigned short dataType) = 0; 
  virtual bool GetLayerOnOff(const char *layer) = 0; 
};

namespace NQckSrvcMgr {

enum 
{
  Err_EXTSRVCEXIST = -128,
  Err_INVINPUT
};

}

class QCKPLUGIN_DLL_SPEC IQckSrvcMgr
{
public:
  virtual IQckWindowSrvc* RegisterForWindowService(int apiVer = QCKPLUGIN_APIVER) = 0;
  virtual IQckMenuSrvc* RegisterForMenuItemService(int apiVer = QCKPLUGIN_APIVER) = 0;
  virtual IQckMarkerSrvc* RegisterForMarkerService(int apiVer = QCKPLUGIN_APIVER) = 0;
  virtual IQckExtractSrvc* RegisterForExtractService(int apiVer = QCKPLUGIN_APIVER) = 0;
  virtual IQckGraphicsSrvc* RegisterForGraphicsService(int apiVer = QCKPLUGIN_APIVER) = 0;
  virtual IQckQuerySrvc* RegisterForQueryService(int apiVer = QCKPLUGIN_APIVER) = 0;
  virtual IQckDatabaseSrvc* RegisterForDatabaseService(int apiVer = QCKPLUGIN_APIVER) = 0;
  virtual IQckLayerSrvc* RegisterForLayerService(int apiVer = QCKPLUGIN_APIVER) = 0;

  virtual int AddExtendedService(const char* ServiceName, void* ServiceHandle) = 0;
  /* NOTE: GetExtendedService must be called during ExecuteQckPlugin function ONLY. */
  virtual void* GetExtendedService(const char* ServiceName) = 0;

  virtual IQckEventNotify* GetNotificationHandler(void* pluginHandle, int notificationType) = 0;

  virtual int GetLastErrorCode() = 0;
  virtual const char* GetLastErrorMsg() = 0;
};

extern "C" {
#endif

int USRPLUGIN_DLL_SPEC InitQckPlugin(const char* PlgPath, void* PlgHandle, void* SrvcMgr);
void USRPLUGIN_DLL_SPEC ExecuteQckPlugin(int Argc, void* Argv);
void USRPLUGIN_DLL_SPEC CloseQckPlugin();

#ifdef __cplusplus 
}
#endif

#endif
